@@ -8,12 +8,22 @@ from itsimodels.core.fields import (
     ListField,
     NumberField,
     StringField,
-    TypeField
+    TypeField, ForeignKeyMultiEmbeddedStr
 )
 from itsimodels.team import GLOBAL_TEAM_KEY
-from itsimodels.core.setup_logging import logger
 
 
+'''
+Many searches in kip embeds various ids in the query string
+we attempt to capture most of them. notice the matching/replacement happens
+in itsicli.content_packs.backup.KeysUpdater
+'''
+GENERIC_SEARCH_RE = (r'id *= *\\"([\w-]+?)(?= |\\)|'             # i.e. serviceid=\"a5d84252-acec-40a4-bf1f-ba6f2aecc6ce\"
+                     r'id *= *([\w-]+?)(?= |$)|'                   # i.e. kpi_id=fcdce583cea75e75d5e26e7d
+                     r'`assess_severity\(([\w-]+?) *, *([\w-]+?)\)`|' # i.e. `assess_severity(a5d84252-acec-40a4-bf1f-ba6f2aecc6ce, fcdce583cea75e75d5e26e7d)`
+                     r'`assess_severity\(([\w-]+?) *, *([\w-]+?) *,.+?\)`|'   # i.e. `assess_severity(a5d84252-acec-40a4-bf1f-ba6f2aecc6ce, fcdce583cea75e75d5e26e7d ,  true, true, true)`
+                     r'`get_full_itsi_summary_[\w]+?\(([\w-]+?)\)`') # i.e. `get_full_itsi_summary_service_health_events(4dbc2a97-8d0e-4b29-92db-2cc5d426a71f)`
+
 class ThresholdLevel(ChildModel):
 
     dynamic_param = NumberField(alias='dynamicParam')
@@ -143,7 +153,7 @@ class Kpi(ChildModel):
 
     backfill_enabled = BoolField(default=False)
 
-    base_search = StringField()
+    base_search = ForeignKeyMultiEmbeddedStr(refers=None, key_regex=GENERIC_SEARCH_RE)
 
     base_search_id = ForeignKey('itsimodels.kpi_base_search.KpiBaseSearch')
 
@@ -183,7 +193,7 @@ class Kpi(ChildModel):
 
     gap_severity_value = StringField(default='-1')
 
-    kpi_base_search = StringField()
+    kpi_base_search = ForeignKeyMultiEmbeddedStr(refers=None, key_regex=GENERIC_SEARCH_RE)
 
     kpi_template_kpi_id = ForeignKey('itsimodels.kpi_threshold_template.KpiThresholdTemplate')
 
@@ -197,29 +207,29 @@ class Kpi(ChildModel):
 
     metric_qualifier = StringField()
 
-    search = StringField()
+    search = ForeignKeyMultiEmbeddedStr(refers=None, key_regex=GENERIC_SEARCH_RE)
 
-    search_aggregate = StringField()
+    search_aggregate = ForeignKeyMultiEmbeddedStr(refers=None, key_regex=GENERIC_SEARCH_RE)
 
-    search_alert = StringField()
+    search_alert = ForeignKeyMultiEmbeddedStr(refers=None, key_regex=GENERIC_SEARCH_RE)
 
     search_alert_earliest = StringField()
 
-    search_alert_entities = StringField()
+    search_alert_entities = ForeignKeyMultiEmbeddedStr(refers=None, key_regex=GENERIC_SEARCH_RE)
 
     search_buckets = StringField()
 
-    search_entities = StringField()
+    search_entities = ForeignKeyMultiEmbeddedStr(refers=None, key_regex=GENERIC_SEARCH_RE)
 
     search_occurrences = NumberField()
 
-    search_time_compare = StringField()
+    search_time_compare = ForeignKeyMultiEmbeddedStr(refers=None, key_regex=GENERIC_SEARCH_RE)
 
-    search_time_series = StringField()
+    search_time_series = ForeignKeyMultiEmbeddedStr(refers=None, key_regex=GENERIC_SEARCH_RE)
 
-    search_time_series_entities = StringField()
+    search_time_series_aggregate = ForeignKeyMultiEmbeddedStr(refers=None, key_regex=GENERIC_SEARCH_RE)
 
-    search_time_series_aggregate = StringField()
+    search_time_series_entities = ForeignKeyMultiEmbeddedStr(refers=None, key_regex=GENERIC_SEARCH_RE)
 
     search_type = StringField(default='adhoc')
 
@@ -291,8 +301,6 @@ class Service(BaseModel):
 
     services_depending_on_me = ListField(ServiceDependency)
 
-    service_id = ForeignKey('itsimodels.service.Service')
-
     service_template_id = ForeignKey('itsimodels.service_template.ServiceTemplate', alias='base_service_template_id')
 
     service_tags = TypeField(ServiceTags)
